import os
import sys
from setuptools import setup, find_packages
from fnmatch import fnmatchcase
from distutils.util import convert_path

def read(fname):
    return open(os.path.join(os.path.dirname(__file__), fname)).read()

standard_exclude = ('*.py', '*.pyc', '*~', '.*', '*.bak', '*.swp*')
standard_exclude_directories = ('.*', 'CVS', '_darcs', os.path.join('.', 'build'), os.path.join('.', 'dist'), 'EGG-INFO', '*.egg-info')
def find_package_data(where='.', package='', exclude=standard_exclude, exclude_directories=standard_exclude_directories):
    out = {}
    stack = [(convert_path(where), '', package)]
    while stack:
        where, prefix, package = stack.pop(0)
        for name in os.listdir(where):
            fn = os.path.join(where, name)
            if os.path.isdir(fn):
                bad_name = False
                for pattern in exclude_directories:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        break
                if bad_name:
                    continue
                if os.path.isfile(os.path.join(fn, '__init__.py')):
                    if not package:
                        new_package = name
                    else:
                        new_package = package + '.' + name
                        stack.append((fn, '', new_package))
                else:
                    stack.append((fn, prefix + name + os.path.sep, package))
            else:
                bad_name = False
                for pattern in exclude:
                    if (fnmatchcase(name, pattern)
                        or fn.lower() == pattern.lower()):
                        bad_name = True
                        break
                if bad_name:
                    continue
                out.setdefault(package, []).append(prefix+name)
    return out

openpyxl_version = "2.5.14" if sys.version.startswith('3.5') else "3.0.0"
twine_version = "1.15.0" if sys.version.startswith('3.5') else "2.0.0"
install_requires = ['docassemble==0.5.73', "3to2==1.1.1", "alembic==1.2.1", "astunparse==1.6.2", "atomicwrites==1.3.0", "attrs==19.3.0", "Babel==2.7.0", "backports.csv==1.0.7", "bcrypt==3.1.7", "beautifulsoup4==4.8.1", "bleach==3.1.0", "blinker==1.4", "cachetools==3.1.1", "certifi==2019.9.11", "cffi==1.13.1", "chardet==3.0.4", "cheroot==8.2.1", "CherryPy==18.3.0", "convertapi==1.1.0", "docutils==0.15.2", "docxcompose==1.0.2", "docxtpl==0.6.3", "et-xmlfile==1.0.1", "fdfgen==0.16.1", "feedparser==5.2.1", "future==0.18.1", "geographiclib==1.50", "geopy==1.20.0", "google-api-python-client==1.7.11", "google-auth==1.6.3", "google-auth-httplib2==0.0.3", "guess-language-spirit==0.5.3", "httplib2==0.14.0", "idna==2.8", "importlib-metadata==0.23", "itsdangerous==1.1.0", "jaraco.functools==2.0", "jdcal==1.4.1", "jellyfish==0.5.6", "Jinja2==2.10.3", "lxml==4.4.1", "Mako==1.1.0", "Markdown==3.1.1", "MarkupSafe==1.1.1", "mdx-smartypants==1.5.1", "more-itertools==7.2.0", "namedentities==1.5.2", "nltk==3.4.5", "numpy==1.17.3", "oauth2client==4.1.3", "openpyxl==" + openpyxl_version, "ordered-set==3.1.1", "packaging==19.2", "pandas==0.25.2", "passlib==1.7.1", "pathlib==1.0.1", "Pattern==3.6", "pdfminer.six==20191020", "pdfminer3k==1.3.1", "phonenumbers==8.10.21", "Pillow==6.2.1", "pkginfo==1.5.0.1", "pluggy==0.13.0", "ply==3.11", "portend==2.5", "py==1.8.0", "pyasn1==0.4.7", "pyasn1-modules==0.2.7", "pycountry==19.8.18", "pycparser==2.19", "pycryptodome==3.9.0", "pycryptodomex==3.9.0", "pycurl==7.43.0.3", "Pygments==2.4.2", "PyJWT==1.7.1", "PyLaTeX==1.3.1", "pyparsing==2.4.2", "pyPdf==1.13", "PyPDF2==1.26.0", "pypdftk==0.4", "Pyphen==0.9.5", "pypng==0.0.20", "pytest==5.2.2", "python-dateutil==2.8.0", "python-docx==0.8.7", "python-editor==1.0.4", "pytz==2019.3", "PyYAML==5.1.2", "pyzbar==0.1.8", "qrcode==6.1", "readme-renderer==24.0", "repoze.lru==0.7", "requests==2.22.0", "requests-toolbelt==0.9.1", "rsa==4.0", "ruamel.yaml==0.16.5", "ruamel.yaml.clib==0.2.0", "scipy==1.3.1", "six==1.12.0", "sortedcontainers==2.1.0", "soupsieve==1.9.4", "SQLAlchemy==1.3.10", "tempora==1.14.1", "textstat==0.5.6", "titlecase==0.12.0", "tqdm==4.36.1", "twilio==6.32.0", "twine==" + twine_version, "tzlocal==2.0.0", "ua-parser==0.8.0", "uritemplate==3.0.0", "urllib3==1.25.6", "us==1.0.0", "user-agents==2.0", "wcwidth==0.1.7", "webencodings==0.5.1", "Werkzeug==0.16.0", "xlrd==1.2.0", "XlsxWriter==1.2.2", "xlwt==1.3.0", "zc.lockfile==2.0", "zipp==0.6.0"]

setup(name='docassemble.base',
      version='0.5.73',
      python_requires='>=3.5',
      description=('The base components of the docassemble system.'),
      long_description=read("README.md"),
      long_description_content_type='text/markdown',
      author='Jonathan Pyle',
      author_email='jhpyle@gmail.com',
      license='MIT',
      url='https://docassemble.org',
      download_url='https://download.docassemble.org/docassemble-base.tar.gz',
      namespace_packages = ['docassemble'],
      install_requires = install_requires,
      packages=find_packages(),
      zip_safe = False,
      package_data=find_package_data(where=os.path.join('docassemble', 'base', ''), package='docassemble.base'),
     )



import tempfile
import subprocess
from PIL import Image, ImageEnhance
from docassemble.base.functions import get_config, get_language, ReturnValue
from docassemble.base.core import DAFile, DAFileList
from PyPDF2 import PdfFileReader
from docassemble.base.logger import logmessage
import pycountry
import sys
import os

def ocr_finalize(*pargs, **kwargs):
    #sys.stderr.write("ocr_finalize started")
    output = dict()
    index = 0
    for parg in pargs:
        #sys.stderr.write("ocr_finalize: index " + str(index) + " is a " + str(type(parg)) + "\n")
        if type(parg) is list:
            for item in parg:
                #sys.stderr.write("ocr_finalize: sub item is a " + str(type(item)) + "\n")
                if type(item) is ReturnValue:
                    output[int(item.value['page'])] = item.value['text']
        else:
            if type(parg) is ReturnValue:
                output[int(parg.value['page'])] = parg.value['text']
        index += 1
    #sys.stderr.write("ocr_finalize: assembling output\n")
    final_output = "\f".join([output[x] for x in sorted(output.keys())])
    #sys.stderr.write("ocr_finalize: final output has length " + str(len(final_output)) + "\n")
    return final_output

def get_available_languages():
    try:
        output = subprocess.check_output(['tesseract', '--list-langs'], stderr=subprocess.STDOUT).decode()
    except subprocess.CalledProcessError as err:
        raise Exception("get_available_languages: failed to list available languages: " + str(err))
    else:
        result = output.splitlines()
        result.pop(0)
        return result
    
def ocr_page_tasks(image_file, language=None, psm=6, x=None, y=None, W=None, H=None, user_code=None, **kwargs):
    #sys.stderr.write("ocr_page_tasks running\n")
    if not (isinstance(image_file, DAFile) or isinstance(image_file, DAFileList)):
        return word("(Not a DAFile or DAFileList object)")
    pdf_to_ppm = get_config("pdftoppm")
    if pdf_to_ppm is None:
        pdf_to_ppm = 'pdftoppm'
    ocr_resolution = get_config("ocr dpi")
    if ocr_resolution is None:
        ocr_resolution = '300'
    langs = get_available_languages()
    if language is None:
        language = get_language()
    if language in langs:
        lang = language
    else:
        ocr_langs = get_config("ocr languages")
        if ocr_langs is None:
            ocr_langs = dict()
        if language in ocr_langs and ocr_langs[language] in langs:
            lang = ocr_langs[language]
        else:
            try:
                pc_lang = pycountry.languages.get(alpha_2=language)
                lang_three_letter = pc_lang.alpha_3
                if lang_three_letter in langs:
                    lang = lang_three_letter
                else:
                    if 'eng' in langs:
                        lang = 'eng'
                    else:
                        lang = langs[0]
                    sys.stderr.write("ocr_file: could not get OCR language for language " + str(language) + "; using language " + str(lang) + "\n")
            except Exception as the_error:
                if 'eng' in langs:
                    lang = 'eng'
                else:
                    lang = langs[0]
                sys.stderr.write("ocr_file: could not get OCR language for language " + str(language) + "; using language " + str(lang) + "; error was " + str(the_error) + "\n")
    if isinstance(image_file, DAFile):
        image_file = [image_file]
    todo = list()
    for doc in image_file:
        if hasattr(doc, 'extension'):
            if doc.extension not in ['pdf', 'png', 'jpg', 'gif']:
                raise Exception("document with extension " + doc.extension + " is not a readable image file")
            if doc.extension == 'pdf':
                #doc.page_path(1, 'page')
                for i in range(PdfFileReader(open(doc.path(), 'rb')).getNumPages()):
                    todo.append(dict(doc=doc, page=i+1, lang=lang, ocr_resolution=ocr_resolution, psm=psm, x=x, y=y, W=W, H=H, pdf_to_ppm=pdf_to_ppm, user_code=user_code))
            else:
                todo.append(dict(doc=doc, page=None, lang=lang, ocr_resolution=ocr_resolution, psm=psm, x=x, y=y, W=W, H=H, pdf_to_ppm=pdf_to_ppm, user_code=user_code))
    #sys.stderr.write("ocr_page_tasks finished\n")
    return todo

def make_png_for_pdf(doc, prefix, resolution, pdf_to_ppm, page=None):
    path = doc.path()
    make_png_for_pdf_path(path, prefix, resolution, pdf_to_ppm, page=page)
    doc.commit()

def make_png_for_pdf_path(path, prefix, resolution, pdf_to_ppm, page=None):
    basefile = os.path.splitext(path)[0]
    test_path = basefile + prefix + '-in-progress'
    with open(test_path, 'a'):
        os.utime(test_path, None)
    if page is None:
        result = subprocess.call([str(pdf_to_ppm), '-r', str(resolution), '-png', str(path), str(basefile + prefix)])
    else:
        result = subprocess.call([str(pdf_to_ppm), '-f', str(page), '-l', str(page), '-r', str(resolution), '-png', str(path), str(basefile + prefix)])
    if os.path.isfile(test_path):
        os.remove(test_path)
    if result > 0:
        raise Exception("Unable to extract images from PDF file")

def ocr_page(doc=None, lang=None, pdf_to_ppm='pdf_to_ppm', ocr_resolution=300, psm=6, page=None, x=None, y=None, W=None, H=None, user_code=None):
    """Runs optical character recognition on an image or a page of a PDF file and returns the recognized text."""
    if page is None:
        page = 1
    sys.stderr.write("ocr_page running on page " + str(page) + "\n")
    the_file = None
    if not hasattr(doc, 'extension'):
        return None
    #sys.stderr.write("ocr_page running with extension " + str(doc.extension) + "\n")
    if doc.extension not in ['pdf', 'png', 'jpg', 'gif']:
        raise Exception("Not a readable image file")
    #sys.stderr.write("ocr_page calling doc.path()\n")
    path = doc.path()
    if doc.extension == 'pdf':
        the_file = None
        if x is None and y is None and W is None and H is None:
            the_file = doc.page_path(page, 'page', wait=False)
        if the_file is None:
            output_file = tempfile.NamedTemporaryFile()
            args = [str(pdf_to_ppm), '-r', str(ocr_resolution), '-f', str(page), '-l', str(page)]
            if x is not None:
                args.extend(['-x', str(x)])
            if y is not None:
                args.extend(['-y', str(y)])
            if W is not None:
                args.extend(['-W', str(W)])
            if H is not None:
                args.extend(['-H', str(H)])
            args.extend(['-singlefile', '-png', str(path), str(output_file.name)])
            result = subprocess.call(args)
            if result > 0:
                return word("(Unable to extract images from PDF file)")
            the_file = output_file.name + '.png'
    else:
        the_file = path
    image = Image.open(the_file)
    color = ImageEnhance.Color(image)
    bw = color.enhance(0.0)
    bright = ImageEnhance.Brightness(bw)
    brightened = bright.enhance(1.5)
    contrast = ImageEnhance.Contrast(brightened)
    final_image = contrast.enhance(2.0)
    file_to_read = tempfile.TemporaryFile()
    final_image.save(file_to_read, "PNG")
    file_to_read.seek(0)
    params = ['tesseract', 'stdin', 'stdout', '-l', str(lang), '--psm', str(psm)]
    sys.stderr.write("ocr_page: piping to command " + " ".join(params) + "\n")
    try:
        text = subprocess.check_output(params, stdin=file_to_read).decode()
    except subprocess.CalledProcessError as err:
        raise Exception("ocr_page: failed to run tesseract with command " + " ".join(params) + ": " + str(err) + " " + str(err.output.decode()))
    sys.stderr.write("ocr_page finished with page " + str(page) + "\n")
    return dict(page=page, text=text)
